@@ -20,6 +20,9 @@ import base64
 import datetime
 import mimetypes
 import html
+import getpass
+import configparser
+import time
 import git
 import markdown
 import cherrypy.process.plugins
@@ -83,7 +86,7 @@ def refresh_repos(param, engine, cachedir, clonedir, refresh, refstatfile): # py
                     if not os.path.isdir(cache):
                         repo = git.Repo.clone_from(url, cache)
                         repo.remotes.origin.fetch()
-                        repo.__del__()
+                        del repo
                     gdir = git.cmd.Git(cache)
                     # Remove local branches that are gone upstream.
                     gdir.fetch(prune=True)
@@ -127,7 +130,7 @@ def refresh_repos(param, engine, cachedir, clonedir, refresh, refstatfile): # py
                 if not os.path.isdir(clone):
                     repo = git.Repo.clone_from(cache, clone)
                     repo.remotes.origin.fetch()
-                    repo.__del__()
+                    del repo
                 else:
                     gdir = git.cmd.Git(clone)
                     gdir.pull()
@@ -355,7 +358,7 @@ def show_history(cwd, display, clonedir): # pylint: disable=too-many-branches, t
             payload = payload + "<td>" + str(commit.author) + "</td>"
             payload = payload + "<td>" + str(formdate(commit)) + "</td>"
         payload = payload + "</table></td>"
-        gdir.__del__()
+        del gdir
         return payload
 
     return "<td valign=top>" + target + "</td>"
@@ -382,3 +385,85 @@ def pull_status(engine, clonedir, refresh, refstatfile):
             "<div style='background-color:#99ffcc'>Refreshing " + status + "...</div>"
 
     return payload
+
+def homesub(value):
+    """ Expand ~ to the full path to the user's home directory. """
+    if '~' in value:
+        value = value.replace('~',os.path.expanduser('~' + getpass.getuser()))
+    return value
+
+def config_handler(configfilename):
+    """ Handle creation and reading of the configuration file. """
+    database = '~/.astblick.db'
+    ipaddr = '127.0.0.1'
+    port = 8080
+    refresh = '900'
+    key = '~/.astblick_key.pem'
+    cert = '~/.astblick_cert.pem'
+    tempdir = '0'
+
+    if not os.path.isfile(configfilename):
+        open(configfilename, 'a', encoding='utf-8').close() # pylint: disable=consider-using-with
+
+    config = configparser.ConfigParser()
+    config.read(configfilename)
+    sections = config.sections()
+
+    if "Options" not in sections:
+        config.add_section("Options")
+        config.set("Options", "database", database)
+        config.set("Options", "ip", ipaddr)
+        config.set("Options", "port", str(port))
+        config.set("Options", "refresh", str(refresh))
+        config.set("Options", "key", str(key))
+        config.set("Options", "cert", str(cert))
+        config.set("Options", "tempdir", str(tempdir))
+        with open(configfilename, 'w', encoding='utf-8') as config.file:
+            config.write(config.file)
+
+    return config
+
+def create_certs(keyf, certf):
+    """ Create self-signed certificates """
+
+    # If key doesn't exist, create, otherwise import.
+    if not os.path.isfile(keyf):
+        key = crypto.PKey()
+        key.generate_key(crypto.TYPE_RSA, 4096)
+        with open(keyf, 'w', encoding='utf-8') as keyfile:
+            keyfile.write(crypto.dump_privatekey(crypto.FILETYPE_PEM, key).decode())
+    else:
+        with open(keyf, 'r', encoding='utf-8') as keyfile:
+            keycontent = keyfile.read()
+        key = crypto.load_privatekey(crypto.FILETYPE_PEM, keycontent)
+
+    # If cert exists, return, otherwise create.
+    if os.path.isfile(certf):
+        return 0
+
+    req = crypto.X509Req()
+    req.get_subject().C = 'US'
+    req.get_subject().ST = 'None'
+    req.get_subject().L = 'None'
+    req.get_subject().O = 'None'
+    req.get_subject().CN = 'localhost'
+    req.set_pubkey(key)
+    req.sign(key, "sha256")
+
+    cert = crypto.X509()
+    cert.get_subject().CN = 'localhost'
+    cert.get_subject().C = 'US'
+    cert.get_subject().ST = 'None'
+    cert.get_subject().L = 'None'
+    cert.get_subject().O = 'None'
+    cert.gmtime_adj_notBefore(0)
+    cert.gmtime_adj_notAfter(31536000)
+    cert.set_issuer(cert.get_subject())
+    cert.set_serial_number(int(time.time()))
+    cert.set_pubkey(key)
+    cert.sign(key, "sha256")
+
+    with open(certf, 'w', encoding='utf-8') as certfile:
+        certfile.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert).decode())
+
+    return 0
